/*	$Id: deSendApp.h,v 1.14 1996/06/24 03:00:53 sam Rel $ */
/*
 * Copyright (c) 1994-1996 Sam Leffler
 * Copyright (c) 1994-1996 Silicon Graphics, Inc.
 * HylaFAX is a trademark of Silicon Graphics
 *
 * Permission to use, copy, modify, distribute, and sell this software and 
 * its documentation for any purpose is hereby granted without fee, provided
 * that (i) the above copyright notices and this permission notice appear in
 * all copies of the software and related documentation, and (ii) the names of
 * Sam Leffler and Silicon Graphics may not be used in any advertising or
 * publicity relating to the software without the specific, prior written
 * permission of Sam Leffler and Silicon Graphics.
 * 
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
 * 
 * IN NO EVENT SHALL SAM LEFFLER OR SILICON GRAPHICS BE LIABLE FOR
 * ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,
 * OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF 
 * LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 
 * OF THIS SOFTWARE.
 */
#ifndef _d2SendApp_
#define	_d2SendApp_
/*
 * IXO Send Page App.
 */
#include "faxApp.h"
#include "ModemServer.h"
#include "FaxTrace.h"
#include "pdsisdn.h"

class UUCPLock;
class fxStackBuffer;

typedef struct
{
  char ack;           /* 'A' oder 'N' */
  char mvp[11];       /* Modified validity period */
  char sm[400];       /* System-Message in IA5 */
  char empf_nr[17];   /* Empfaenger-Nummer */
  char empf_zeit[13]; /* Uhrzeit des Empfangs */
  int  ec;            /* error-code */
} ack_typ;

typedef struct
{
  char empf[17];      /* Empfaenger-Nummer */
  char abs[17];       /* Absender-Nummer */
  char auth[200];     /* authentifications-code (momentan nicht verwendet) */
  int  mess_type;      /* message type */
  char message[500];  /* Numerische Message */
} call_in_typ;


typedef struct
{
 int         trn;  /* transaction reference number */
 char        op;   /* operation, 'O' oder 'R' */
 int         ot;   /* opertion type siehe OT_... */
 ack_typ     ack;  /* Werte des ACK-Paketes */
 call_in_typ cin;  /* Werte der Notification */
} pack_typ;

class d2SendApp : public ModemServer, public faxApp {
public:
    struct stringtag {
	const char*	 name;
	fxStr d2SendApp::* p;
	const char*	 def;		// NULL is shorthand for ""
    };
    struct numbertag {
	const char*	 name;
	u_int d2SendApp::*p;
	u_int		 def;
    };
private:
// runtime state
    fxBool	ready;			// modem ready for use
    UUCPLock*	modemLock;		// uucp lockfile handle
    time_t	connTime;		// time connected to peer

    fxStr	pagerSetupCmds;		// pager-specific modem setup commands
    fxStr	pagerTTYParity;		// parity setting for tty
    u_int	pagerMaxMsgLength;	// default max message text length

    fxStr	ixoService;		// IXO protocol service string
    fxStr	ixoDeviceID;		// IXO protocol device id string
    u_int	ixoMaxUnknown;		// max unknown responses to permit
    u_int	ixoIDProbe;		// time between probes for ID= string
    u_int	ixoIDTimeout;		// timeout waiting for initial ID=
    u_int	ixoLoginRetries;	// # times to retry login procedure
    u_int	ixoLoginTimeout;	// timeout on login procedure
    u_int	ixoGATimeout;		// timeout waiting for go-ahead message
    u_int	ixoXmitRetries;		// # times to retry sending msg block
    u_int	ixoXmitTimeout;		// timeout waiting for xmit response
    u_int	ixoAckTimeout;		// timeout waiting ro final ack/nak

    static d2SendApp* _instance;

    static const stringtag strings[];
    static const stringtag atcmds[];
    static const numbertag numbers[];

// configuration support
    void	setupConfig();
    void	resetConfig();
    fxBool	setConfigItem(const char* tag, const char* value);
    u_int	getConfigParity(const char* value) const;
// IXO transmission support
    int send_d2_message(sms_typ *sms);
    CallStatus open_smsc(sms_typ *sms);
    int rcv_packet(char *packet, sms_typ *sms, int timeout);
    int rcv_packet(char *packet, char *pattern, sms_typ *sms, int timeout);
    int close_smsc(sms_typ *sms);
    int send_transfer_req(sms_typ *sms);
    int rcv_string_packet(char *packet, char pattern ,sms_typ *sms, int timeout, int maxlen);
    int checksum(char *buf);
    int isinstr(char *s1, char *s2);
    void isdn_sleep(int sec);
    int send_alert(sms_typ *sms);
    char *prep_save_time(int st);
    char *ascii_to_ia5(char *message);
    char *ia5_to_ascii(char *ia5);
    int decode_packet (char     *packet, pack_typ *pkt);
    char *decode_ec (int ec);
    char *timestamp_to_timestr(char *ts);
    int anforderung_channeld(char *);
    void freigabe_channeld(char *);
    char *decode_mvp(char *mvp);
    void	sendPage(FaxRequest&, FaxMachineInfo&);
    void	sendPage(FaxRequest&, FaxMachineInfo&, const fxStr&,
		    const fxStr&);
    fxBool	sendPagerMsg(FaxRequest&, faxRequest&, const fxStr&, fxStr&);
    u_int	getResponse(fxStackBuffer& buf, long secs);
    fxBool	prepareMsg(FaxRequest&, FaxMachineInfo&, fxStr&);
    fxBool	pagePrologue(FaxRequest&, const FaxMachineInfo&, fxStr&);
    fxBool	pageGoAhead(FaxRequest&, const FaxMachineInfo&, fxStr&);
    fxBool	pageEpilogue(FaxRequest&, const FaxMachineInfo&, fxStr&);
    void	sendFailed(FaxRequest&, FaxSendStatus, const char*, u_int = 0);
    void	notifyPageSent(FaxRequest& req, u_int fi);
    time_t	getConnectTime() const;
// modem handling
    fxBool	lockModem();
    void	unlockModem();
    fxBool	setupModem();
// notification interfaces used by ModemServer
    void	notifyModemReady();
    void	notifyModemWedged();
// miscellaneous
    fxBool	putModem(const void* data, int n, long ms = 0);
    void	traceResponse(const fxStackBuffer& buf);
    void	traceIXO(const char* fmt ...);
    void	traceIXOCom(const char* dir, const u_char* data, u_int cc);
public:
    d2SendApp(const fxStr& device, const fxStr& devID);
    ~d2SendApp();

    static d2SendApp& instance();

    void	initialize(int argc, char** argv);
    void	open();
    void	close();

    FaxSendStatus send(const char* filename);

    fxBool	isReady() const;
};
inline fxBool d2SendApp::isReady() const 	{ return ready; }
#endif
